#include <math.h>
#include "colorizer.h"

static RGBColor interpolate(RGBColor color1, RGBColor color2, double t) {
   return (RGBColor) {
      .red = (color1.red + (color2.red - color1.red) * t),
      .green = (color1.green + (color2.green - color1.green) * t),
      .blue = (color1.blue + (color2.blue - color1.blue) * t)
   };
}

static RGBColor get_color_in_scale(double value,
      RGBColor* colors, unsigned int nof_colors,
      double color_shift, double color_progression) {
   double t = fmod(color_shift + value *
      color_progression, (double) nof_colors);
   unsigned int cp1 = floor(t); t = t - cp1;
   unsigned int cp2 = (cp1 + 1) % nof_colors;
   RGBColor color = interpolate(colors[cp1], colors[cp2], t);
   return color;
}

RGBColor get_color(Colorizer* colorizer,
      unsigned int iterations, unsigned int maxiterations, double t) {
   return get_color_in_scale(
      log(iterations + t) / log(maxiterations),
      colorizer->colors, colorizer->nof_colors,
      colorizer->shift, colorizer->progression);
}
