#include <math.h>
#include "mandelbrot.h"

/* Algorithm based on pseudo-code at
   https://en.wikipedia.org/wiki/Mandelbrot_set#Continuous_.28smooth.29_coloring
*/
RGBColor mandelbrot(double c_x, double c_y, unsigned int maxiterations,
      Colorizer* colorizer) {
   const double bailout = 2 << 8;
   const double bailout2 = bailout * bailout;
   const double log_bailout = log(bailout);
   double x = 0;
   double y = 0;
   for (unsigned int iteration = 0; iteration < maxiterations; ++iteration) {
      double x_square = x * x;
      double y_square = y * y;
      double distance2 = x_square + y_square;
      if (distance2 >= bailout2) {
	 double distance = sqrt(distance2);
	 double nu = log(log(distance) / log_bailout) / M_LN2;
	 if (nu > 1) nu = 1;
	 return get_color(colorizer, iteration, maxiterations, 1 - nu);
      }
      double next_x = x_square - y_square + c_x;
      double next_y = 2 * x * y + c_y;
      x = next_x; y = next_y;
   }
   return (RGBColor){0, 0, 0};
}
